program compute_pi
implicit none 

! times using cpu_time
real t0,t1

integer            :: i
integer, parameter :: n=100000000
real(kind=8), parameter :: pi_ref = 3.1415926535897932384626433_8
real(kind=8)       :: w,x,sum,pi,f,a

! additional variables for the manual CAF worksharing:
integer            :: myrank, numimages
integer            :: ibegin, iend, ichunk
! distributed array for the partial result of each image:
real(kind=8)       :: mypi[*]

myrank    = THIS_IMAGE()
numimages = NUM_IMAGES()

write (*,*) 'I am image number',myrank,' of ',numimages,' images'

call cpu_time(t0)

! calculate pi = integral [0..1] 4/(1+x**2) dx
w=1.0_8/n
sum=0.0_8

ichunk=(n-1)/numimages+1 ! = n/numimages rounded up
ibegin=1+ichunk*(myrank-1)
iend  =min(ichunk*myrank, n) 
! do i=ibegin,iend
! or alternatively:
do i=1+(myrank-1), n, numimages
     x=w*(i-0.5_8)
!    function to integrate
     sum=sum + 4.0_8/(1.0_8+x*x)
enddo
mypi=w*sum
! which is semantically identical to:
! mypi[myrank]=w*sum

SYNC ALL

if (myrank == 1) then
  pi=0.0
  do i=1,numimages
    pi = pi+mypi[i]
  end do
end if  

call cpu_time(t1)

if (myrank == 1) then
  write (*,'(a,f24.16)')   'computed  pi     = ', pi
  write (*,'(a,f24.16)')   'reference pi_ref = ', pi_ref
  write (*,'(a,f24.16)')   'error= pi-pi_ref = ', pi - pi_ref
  write (*,'(a,f14.6,a)')  'cpu_time:          ', t1-t0, ' sec'
end if  

end program compute_pi
