#include <upc.h>
#include <upc_collective.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>

const int n = 100000000;

const double pi_ref = 3.1415926535897932384626433;

int main(int argc, char** argv)
{
  int i;
  double w,x,sum,pi;
  clock_t t1,t2;
  struct timeval tv1,tv2; struct timezone tz;
  /* additional variables for the manual CAF worksharing: */
  int ichunk, ibegin, iend; 
  upc_lock_t *lock;
  static shared double pi0; /* upc: located in thread 0, accessible by all threads */

  printf("I am thread number %d of %d threads\n", MYTHREAD, THREADS);

  lock = upc_all_lock_alloc();
  gettimeofday(&tv1, &tz);
  t1=clock();
 
/* calculate pi = integral [0..1] 4/(1+x**2) dx */
  w=1.0/n;
  sum=0.0;
  ichunk=(n-1)/THREADS+1;  /* = n/THREADS rounded up */
  ibegin=1+ichunk*(MYTHREAD);
  iend  =ibegin+ichunk-1; 
  if(iend>n) iend=n;
  /* for (i=ibegin;i<=iend;i++) */
  /* or alternatively: */
  for (i=1+MYTHREAD;i<=n;i=i+THREADS)
  {
    x=w*((double)i-0.5);
    sum=sum+4.0/(1.0+x*x);
  }

  if (MYTHREAD==0) pi0=0;
  upc_barrier;
  upc_lock(lock);
    pi0 = pi0 + w*sum; /* executed by each thread */
  upc_unlock(lock);
  upc_barrier;
  pi = pi0;

  t2=clock();
  gettimeofday(&tv2, &tz);
  if (MYTHREAD==0)
  {
    printf( "computed  pi     = %19.16f\n", pi );
    printf( "reference pi_ref = %19.16f\n", pi_ref );
    printf( "error= pi-pi_ref = %19.16f\n", pi - pi_ref );
    printf( "CPU time (clock)                = %12.6f sec\n", (t2-t1)/1000000.0 );
    printf( "wall clock time (gettimeofday)  = %12.6f sec\n", (tv2.tv_sec-tv1.tv_sec) + (tv2.tv_usec-tv1.tv_usec)*1e-6 );
  }
  return 0;
}
